﻿using Performance.DtoModels;
using Performance.EntityModels;
using Performance.Infrastructure;
using Performance.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Performance.Services
{
    public class AssessService : IAutoInjection
    {
        private PerforAsassessRepository perforAsassessRepository;
        private PerforAsdataRepository perforAsdataRepository;
        private PerforAscolumnsRepository perforAscolumnsRepository;
        private PerforAstempassessRepository perforAstempassessRepository;
        private PerforAstempcolumnsRepository perforAstempcolumnsRepository;
        private PerforPerallotRepository perforPerallotRepository;
        private PerforImemployeeRepository perforImemployeeRepository;
        public AssessService(PerforAsassessRepository perforAsassessRepository,
            PerforAsdataRepository perforAsdataRepository,
            PerforAscolumnsRepository perforAscolumnsRepository,
            PerforAstempassessRepository perforAstempassessRepository,
            PerforAstempcolumnsRepository perforAstempcolumnsRepository,
            PerforPerallotRepository perforPerallotRepository,
            PerforImemployeeRepository perforImemployeeRepository)
        {
            this.perforAsassessRepository = perforAsassessRepository;
            this.perforAsdataRepository = perforAsdataRepository;
            this.perforAscolumnsRepository = perforAscolumnsRepository;
            this.perforAstempassessRepository = perforAstempassessRepository;
            this.perforAstempcolumnsRepository = perforAstempcolumnsRepository;
            this.perforPerallotRepository = perforPerallotRepository;
            this.perforImemployeeRepository = perforImemployeeRepository;
        }

        #region 考核类别
        /// <summary>
        /// 考核类别列表
        /// </summary>
        /// <param name="allotID"></param>
        /// <returns></returns>
        public ApiResponse AssessList(int allotID)
        {
            var list = perforAsassessRepository.GetEntities(t => t.AllotID == allotID);
            return new ApiResponse(ResponseType.OK, list);
        }

        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="allotID"></param>
        /// <param name="assessName"></param>
        /// <returns></returns>
        public ApiResponse AddAssess(int allotID, string assessName)
        {
            var entity = perforAsassessRepository.GetEntities(t => t.AllotID == allotID && t.AssessName == assessName);
            if (entity != null && entity.Count > 0)
                return new ApiResponse(ResponseType.Fail, "考核类别重复");
            var assess = new as_assess { AllotID = allotID, AssessName = assessName };
            var result = perforAsassessRepository.Add(assess);
            return result ? new ApiResponse(ResponseType.OK, assess) : new ApiResponse(ResponseType.Fail);
        }

        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="assessID"></param>
        /// <param name="assessName"></param>
        /// <returns></returns>
        public ApiResponse EditAssess(int assessID, string assessName)
        {
            var entity = perforAsassessRepository.GetEntity(t => t.ID == assessID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核类别ID无效");
            entity.AssessName = assessName;

            var result = perforAsassessRepository.Update(entity);
            return result ? new ApiResponse(ResponseType.OK, entity) : new ApiResponse(ResponseType.Fail);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="assessID"></param>
        /// <returns></returns>
        public ApiResponse DelAssess(int assessID)
        {
            var entity = perforAsassessRepository.GetEntity(t => t.ID == assessID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核类别ID无效");

            var result = perforAsassessRepository.Remove(entity);
            var columns = perforAscolumnsRepository.GetEntities(t => t.AssessID == assessID);
            if (result && columns != null && columns.Count > 0)
                result = perforAscolumnsRepository.RemoveRange(columns.ToArray());
            return result ? new ApiResponse(ResponseType.OK) : new ApiResponse(ResponseType.Fail);
        }

        /// <summary>
        /// 科室考核类别归纳
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public ApiResponse SetAssessType(SetAssessRequest request)
        {
            var entity = perforAsassessRepository.GetEntity(t => t.ID == request.AssessID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核类别ID无效");

            var entities = perforAsdataRepository.GetEntities(t => t.AllotID == request.AllotID);
            if (entities != null)
            {
                //保留共有
                var intersectList = entities.Where(t => t.AssessID == request.AssessID).Select(t => t.Department).Intersect(request.Department).ToList();
                if (intersectList != null && intersectList.Count() > 0)
                    intersectList.ForEach(t => request.Department.Remove(t));
                //删除更改
                var removeList = perforAsdataRepository.GetEntities(t => !intersectList.Contains(t.Department));
                if (removeList != null && removeList.Count() > 0)
                    perforAsdataRepository.RemoveRange(removeList.ToArray());
            }
            //添加新增
            var addList = request.Department.Select(t => new as_data { AllotID = request.AllotID, AssessID = request.AssessID, Department = t }).ToArray();
            var result = perforAsdataRepository.AddRange(addList);

            return result ? new ApiResponse(ResponseType.OK) : new ApiResponse(ResponseType.Fail);
        }
        #endregion

        #region 考核项
        /// <summary>
        /// 列头列表
        /// </summary>
        /// <param name="assessID"></param>
        /// <returns></returns>
        public ApiResponse ColumnList(int assessID)
        {
            var list = perforAscolumnsRepository.GetEntities(t => t.AssessID == assessID).OrderBy(t => t.ParentID).ThenBy(t => t.Sort);
            return new ApiResponse(ResponseType.OK, list);
        }

        /// <summary>
        /// 新增考核项
        /// </summary>
        /// <param name="assessID"></param>
        /// <param name="parentID"></param>
        /// <param name="columnName"></param>
        /// <returns></returns>
        public ApiResponse AddColumn(int assessID, int parentID, string columnName, int sort)
        {
            var entity = perforAsassessRepository.GetEntity(t => t.ID == assessID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核类别ID无效");
            if (parentID > 0)
            {
                var centity = perforAscolumnsRepository.GetEntity(t => t.ID == parentID);
                if (centity == null)
                    return new ApiResponse(ResponseType.Fail, "考核项目父级ID无效");
            }
            var column = new as_columns { AssessID = assessID, ParentID = parentID, ColumnName = columnName, Sort = sort };
            var result = perforAscolumnsRepository.Add(column);
            return result ? new ApiResponse(ResponseType.OK, column) : new ApiResponse(ResponseType.Fail);
        }

        /// <summary>
        /// 修改考核项
        /// </summary>
        /// <param name="columnID"></param>
        /// <param name="columnName"></param>
        /// <returns></returns>
        public ApiResponse EditColumn(int columnID, string columnName, int sort)
        {
            var entity = perforAscolumnsRepository.GetEntity(t => t.ID == columnID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核项目父级ID无效");
            entity.ColumnName = columnName;
            entity.Sort = sort;
            var result = perforAscolumnsRepository.Update(entity);
            return result ? new ApiResponse(ResponseType.OK, entity) : new ApiResponse(ResponseType.Fail);
        }

        /// <summary>
        /// 删除考核项
        /// </summary>
        /// <param name="columnID"></param>
        /// <returns></returns>
        public ApiResponse DelColumn(int columnID)
        {
            var entity = perforAscolumnsRepository.GetEntity(t => t.ID == columnID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核项目父级ID无效");

            var result = perforAscolumnsRepository.Remove(entity);
            return result ? new ApiResponse(ResponseType.OK) : new ApiResponse(ResponseType.Fail);
        }
        #endregion

        #region 考核数据

        /// <summary>
        /// 考核数据列表
        /// </summary>
        /// <param name="assessID"></param>
        /// <returns></returns>
        public ApiResponse DataList(int assessID)
        {
            var entity = perforAsassessRepository.GetEntity(t => t.ID == assessID);
            if (entity == null)
                return new ApiResponse(ResponseType.Fail, "考核类别ID无效");

            var list = perforAsdataRepository.GetEntities(t => t.AssessID == assessID);
            if (list != null)
            {
                var response = list.Select(t => new AssessDataResponse
                {
                    ID = t.ID,
                    AssessID = t.AssessID,
                    Department = t.Department,
                    AssessData = string.IsNullOrEmpty(t.RowData)
                        ? new List<AssessData>()
                        : JsonHelper.Deserialize<List<AssessData>>(t.RowData)
                });
                return new ApiResponse(ResponseType.OK, response);
            }
            return new ApiResponse(ResponseType.OK);
        }

        /// <summary>
        /// 编辑考核数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public ApiResponse EditAssessData(List<AssessRow> request)
        {
            var dataIDList = request.Select(t => t.DataID).ToList();
            var dataList = perforAsdataRepository.GetEntities(t => dataIDList.Contains(t.ID));
            foreach (var item in request)
            {
                var entity = dataList.FirstOrDefault(t => t.ID == item.DataID);
                if (entity != null)
                {
                    entity.RowData = JsonHelper.Serialize(item.AssessData);
                    perforAsdataRepository.Update(entity);
                }
            }
            return new ApiResponse(ResponseType.OK);
        }
        #endregion

        #region 模版
        /// <summary>
        /// 考核模版列表
        /// </summary>
        /// <param name="allotID"></param>
        /// <returns></returns>
        public ApiResponse TempAssessList()
        {
            var list = perforAstempassessRepository.GetEntities();
            return new ApiResponse(ResponseType.OK, list);
        }

        /// <summary>
        /// 模版列头列表
        /// </summary>
        /// <param name="assessID"></param>
        /// <returns></returns>
        public ApiResponse TempColumnList(int assessID)
        {
            var list = perforAstempcolumnsRepository.GetEntities(t => t.AssessID == assessID)?.OrderBy(t => t.ParentID).ThenBy(t => t.Sort);
            return new ApiResponse(ResponseType.OK, list);
        }

        /// <summary>
        /// 使用模版
        /// </summary>
        /// <returns></returns>
        public ApiResponse UseTemplate(int allotID, int assessID)
        {
            var allot = perforPerallotRepository.GetEntity(t => t.ID == allotID);
            var temp = perforAstempassessRepository.GetEntity(t => t.ID == assessID);
            if (temp != null && allot != null)
            {
                string assessName = $"{temp.AssessName}A{allotID}-{allot.Year}年{allot.Month}月";
                var assess = perforAsassessRepository.GetEntity(t => t.AssessName == assessName && t.AllotID == allotID);
                if (assess != null)
                    return new ApiResponse(ResponseType.Fail, "该模版已经被使用");
                assess = new as_assess
                {
                    AllotID = allotID,
                    AssessName = assessName
                };
                //copy模版信息
                var result = perforAsassessRepository.Add(assess);

                var tempColumns = perforAstempcolumnsRepository.GetEntities(t => t.AssessID == assessID);
                var columns = tempColumns?.Select(t => new as_columns
                {
                    AssessID = assess.ID,
                    ParentID = t.ParentID,
                    ColumnName = t.ColumnName,
                    Sort = t.Sort
                }).ToList();
                //copy模版列头
                if (result && columns != null && columns.Count > 0)
                    result = perforAscolumnsRepository.AddRange(columns.ToArray());
                return result ? new ApiResponse(ResponseType.OK) : new ApiResponse(ResponseType.Fail);
            }
            return allot == null ? new ApiResponse(ResponseType.Fail, "绩效ID无效") : new ApiResponse(ResponseType.Fail, "模版信息不存在");
        }
        #endregion

        /// <summary>
        /// 科室列表
        /// </summary>
        /// <returns></returns>
        public List<TitleValue> Department(int allotId)
        {
            var allot = perforPerallotRepository.GetEntity(t => t.ID == allotId);
            if (allot != null)
            {
                var _checked = perforAsdataRepository.GetEntities(t => t.AllotID == allotId)?.Select(t => t.Department).Distinct();
                var allotList = perforPerallotRepository.GetEntities(t => t.HospitalId == allot.HospitalId);
                var idList = allotList.Select(s => s.ID).ToList();
                var department = perforImemployeeRepository.GetEntities(t => t.Department != "" && idList.Contains(t.AllotID.Value));
                if (department != null && department.Count > 0)
                {
                    var departmentList = department.Select(t => t.Department).Distinct();
                    departmentList = departmentList.Except(_checked).ToList();
                    return departmentList.Select(t => new TitleValue { Title = t, Value = t }).OrderBy(t => t.Title).ToList();
                }
            }
            return new List<TitleValue>();
        }
    }
}
